/*
 *   Copyright 2009 Andrew Stromme <astromme@chatonka.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef LISTSOURCE_H
#define LISTSOURCE_H

#include <Plasma/DataContainer>

#include <rtm/list.h>

class ListSource : public Plasma::DataContainer
{
Q_OBJECT
public:
    ListSource(RTM::ListId id, RTM::Session* session, QObject* parent);
    ~ListSource();
  
public slots:
    void updateRequest(DataContainer *source);
    void update();

private:
  RTM::ListId id;
  RTM::Session *session;
  RTM::List *list;
};

#endif // LISTSOURCE_H
