/** Stylesheet for the UserWidget used in the openDesktop applet

    Please use the following placeholders for colors and sizes.
    They are replaced by colors matching the theme.

        %textcolor      : Foreground color, used for text
        %background     : background color, defaults to transparant
        %visitedlink    : visited link color
        %activatedlink  : activated link color
        %hoveredlink    : link color when hovered
        %smallfontsize  : Size of the smallest readable font
        %fontsize       : Normal size for fonts
 */

body {
    margin: 0px 0px 0px 0px;
    color: %textcolor;
}

h2 {
    font-size: 10px;
    color: %textcolor;
    opacity: .4;
}

td {
    font-size: %smallfontsize;
    font-style: normal;
}

.infoheader {
    font-weight: bold;
    text-align: right;
}

.rowheader {
    font-size: %smallfontsize;
    vertical-align: top;
    font-weight: bold;
    text-align: right;
    opacity: .7;
}

a:visited   { color: %visitedlink; }
a:link   { color: %link .8; }
a:hover { text-decoration: none; opacity: .4; }
