/*
 * Copyright 2008  Petri Damsten <damu@iki.fi>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef ADDMACRO_H
#define ADDMACRO_H

#include <pastemacroexpander.h>
#include <KDialog>

class KComboBox;
class QVBoxLayout;

class AddMacro : public KDialog
{
    Q_OBJECT
public:
    AddMacro(QWidget* parent);
    ~AddMacro();

    QString macro();

protected slots:
    void currentIndexChanged(int index);

private:
    KComboBox* m_macrosComboBox;
    QVBoxLayout* m_layout;
    QWidget* m_widget;
    QWidget* m_params;
};

#endif
